#!/usr/bin/env python
# -*- coding: utf8 -*-

import cgi
import os
import sys
import re

__author__ = "Dirk Straube"
__license__ = "GPL"
__version__ = "0.1"
__license__ = "MIT"
__email__ = "dirk@brainbombs.de"


class TranscriptParser:

    transcript_file = ""
    content = ""
    formatted_content = ""
    header = ""
    header_file = "head.html"
    game_name = ""
    footer = "\n</div>\n</body>\n</html>"
    tags = {'command': '>',
            'comment': '#',
            'transcript': 'Start of a transcript of'}
    directions = ['n', 'north', 'ne', 'northeast', 'nw', 'northwest',
        's', 'south', 'se', 'southeast', 'sw', 'southwest', 'w', 'west',
        'e', 'east', 'up', 'u', 'down', 'd', 'leave']
    room_commands = ['look', 'l']
    room_nav = ""

    def __init__(self, transcript_file=None):
        if os.path.exists(self.header_file) is False:
            # if the header file isn't found, try to use the script file
            # location as a full path
            self.header_file = "%s%s%s" % (
                os.path.dirname(os.path.abspath(__file__)), os.sep,
                self.header_file)
        self.header = self.load_file(self.header_file)
        if transcript_file:
            self.transcript_file = transcript_file
            self.content = self.load_file(transcript_file)
            if len(self.content) > 0:
                self.parse_if_text()

    def escape_html(self, text):
        """Escape a string to display as HTML
        """
        return cgi.escape(text, quote=True).\
               replace('\n', '<br>\n').\
               replace('\t', '&emsp;').\
               replace('  ', ' &nbsp;')

    def load_file(self, file_name):
        """Load a file and return it's content

        Keyword arguments:
        file_name -- name or path of the file

        return None if an error occurred
        """
        content = ""
        try:
            if os.path.exists(file_name):
                content = open(file_name).read()
            else:
                print(("Error: Can't locate file '%s'." % (file_name, )))
            return content
        except Exception as exc:
            print(("Error: Failed to load file '%s': %s" % (file_name, exc)))
            return content

    def write_html_file(self, file_name):
        """Save HTML content into a file

        Keyword arguments:
        file_name -- name or path of the file

        return False if an error occurred or there is nothing to output,
        True otherwise
        """
        try:
            if len(self.formatted_content) == 0:
                return False
            out_file = open(file_name, "w")
            out_file.write(self.get_html())
            out_file.close()
            return True
        except Exception as exc:
            print(("Error: Failed to write file '%s': %s" % (file_name, exc)))
            return False

    def remove_tag(self, raw_html):
        """removes all HTML tags from a string

        will be used to remove the newlines and other content that we do not
        need here.
        """
        return re.sub(r'<.*?>', '', raw_html).strip()

    def get_html(self):
        """return the formatted output as string
        """
        if len(self.formatted_content) == 0:
            return ""
        if len(self.game_name) > 0:
            title = "<h1>Transcript for \"%s\"</h1>" % \
                (self.escape_html(self.game_name), )
        else:
            title = "<h1>Transcript Test Report</h1>"
        return "%s\n%s\n<div class=\"content\">%s\n%s\n%s" % \
            (self.header, title, self.room_nav,
             self.formatted_content, self.footer)

    def parse_if_text(self, content=None):
        """parse the transcript and turn it into formatted output
        """
        if not content:
            content = self.content
        if len(content) == 0:
            return False
        content = self.escape_html(content)
        new_content = ''
        found_note = False
        found_game_name = False
        new_room = 0
        this_room = ""
        link_counter = 1
        self.room_nav = '<div class="roomnav">\nRoom Navigation<ul>\n'
        check_command = self.escape_html(self.tags['command'])
        check_comment = self.escape_html(self.tags['command'] +
                        self.tags['comment'])
        check_transcript = self.escape_html(self.tags['transcript'])

        # Process the content
        for line in content.split("\n"):
            print_line = True
            if line.startswith(check_comment):
                # this is a comment, put command into comment line
                found_note = True
                new_content += '<div class="note">' + \
                               line[len(check_comment):] + '</div>\n'
                print_line = False
            elif line.startswith(check_command):
                new_room = 0
                found_note = False
                new_content += '<div class="command">' + line + '</div>\n'
                print_line = False
                cmd = self.remove_tag(line[len(check_command):].strip().lower())
                # check if we have a command that allows to change location
                if cmd in self.directions:
                    new_room = 1
                # check if we have a command that allows us to identify the
                # room when we don't know where we are yet
                if cmd in self.room_commands and this_room == "":
                    new_room = 2
            elif line.startswith(check_transcript) and len(self.game_name) == 0:
                found_game_name = True
            else:
                if found_note is True:
                    # ignore this line, the command after the comment usually
                    # leads to an interpreter error
                    print_line = False
                elif found_game_name is True:
                    # we found the transcript code, so we can now detect the
                    # name of the game this transcript belongs to
                    if len(self.remove_tag(line)) > 0:
                        self.game_name = self.remove_tag(line)
                        found_game_name = False
                else:
                    if new_room > 0:
                        if line.startswith('('):
                            # ignore this line, it's a condition'
                            pass
                        elif len(self.remove_tag(line)) == 0:
                            # first line needs to be empty,
                            # if it isn't this is not a room change
                            new_room = 2
                        else:
                            if new_room != 2:
                                new_room = 0
                        if new_room == 2 and len(self.remove_tag(line)) > 0:
                            new_room = False
                            # extract room name
                            this_room = self.remove_tag(line)
                            new_content += '<div class="room" id="link' + \
                                str(link_counter) + '">' + line + "</div>\n"
                            self.room_nav += '<li><a href="#link' + \
                                str(link_counter) + '">' + \
                                self.escape_html(this_room) + '</a></li>\n'
                            print_line = False
                            link_counter += 1
            if print_line:
                new_content += line + "\n"
        self.room_nav += "</ul>\n</div>"
        self.formatted_content = new_content
        return (len(self.formatted_content) > 0)


def main():
    """Main function, does all the work

    Parses the command line parameters, processes the transcript and
    outputs the formatted result as HTML file. Either via stdout ot into a
    file, depending on the command line parameters.
    """
    if len(sys.argv) > 1:

        if_parser = TranscriptParser(sys.argv[1])
        if len(sys.argv) > 2:
            if if_parser.write_html_file(sys.argv[2]):
                print(("HTML file successfully written into '%s'" %
                    (sys.argv[2], )))
            else:
                print("Script aborted.")
        else:
            print((if_parser.get_html()))
    else:
        # show help
        script_name = os.path.basename(sys.argv[0])
        print(("%s Version %s by %s(%s)\n" % (script_name, __version__,
            __author__, __email__)))
        print(("Usage: %s <transcript_file> [output_file]" % (script_name, )))

if __name__ == '__main__':
    main()
